@@ -5,12 +5,12 @@
 use strict;
 use Pod::Usage;
 use Getopt::Long;
-use Bio::ToolBox::data_helper qw(parse_list);
 use Bio::ToolBox::file_helper qw(
 	open_to_read_fh
 	open_to_write_fh
 );
-my $VERSION = 1.14;
+use Bio::ToolBox::utility;
+my $VERSION = 1.20;
 
 print "\n A tool for switching the order of cluster files\n";
 
