@@ -8,21 +8,13 @@ use Pod::Usage;
 use FindBin qw($Bin);
 use Statistics::Lite qw(sum min max mean median stddevp);
 use Statistics::Descriptive;
-use Bio::ToolBox::data_helper qw(
-	find_column_index
-	format_with_commas
-	splice_data_structure
-);
+use Bio::ToolBox::Data; 
 use Bio::ToolBox::db_helper qw(
 	open_db_connection
 	verify_or_request_feature_types
-	get_feature
-	get_region_dataset_hash
-);
-use Bio::ToolBox::file_helper qw(
-	load_tim_data_file 
-	write_tim_data_file 
 );
+use Bio::ToolBox::utility;
+
 my $parallel;
 eval {
 	# check for parallel support
@@ -42,7 +34,7 @@ my $PARAMETRIC    = 1;
 my $ORDINAL       = 0;
 	
 use constant LOG10 => log(10);
-my $VERSION = '1.17';
+my $VERSION = '1.20';
 
 print "\n This program will correlate positions of occupancy between two datasets\n\n";
 
@@ -124,24 +116,30 @@ check_defaults();
 
 
 ### Open input file
-my $mainData = load_tim_data_file($infile) or 
+my $Data = Bio::ToolBox::Data(file => $infile) or 
 	die " unable to open input file '$infile'!\n";
 
 
 
 ### Open database connection
-my ($db, $ddb);
-unless ($database) {
-	$database = $mainData->{'db'} || undef;
-}
 if ($database) {
-	$db = open_db_connection($database) or 
-		die " unable to open database '$database' connection!\n";
+	# update or add database as required
+	if ($Data->database and $database ne $Data->database) {
+		# update with new database
+		printf " updating main database name from '%s' to '%s'\n", 
+			$Data->database, $database;
+		print "   Re-run without --db option if you do not want this to happen\n";
+		$Data->database($database);
+	}
+	elsif (not $Data->database) {
+		$Data->database($database);
+	}
 }
-if ($data_database) {
-	# separate database for the refDataSet and testDataSet
+my $ddb;
+if (defined $data_database) {
+	# specifically defined a data database
 	$ddb = open_db_connection($data_database) or 
-		die " unable to open data database '$data_database' connection!\n";
+		die "unable to establish data database connection to $data_database!\n";
 }
 
 
@@ -150,16 +148,12 @@ if ($data_database) {
 validate_or_request_dataset();
 
 
-### Identify column indices
-my %index = identify_indices();
-
-
 
 ### Collect the data correlations
 my $start_time = time;
 # check whether it is worth doing parallel execution
 if ($cpu > 1) {
-	while ($cpu > 1 and $mainData->{'last_row'}/$cpu < 100) {
+	while ($cpu > 1 and $Data->last_row / $cpu < 100) {
 		# I figure we need at least 100 lines in each fork split to make 
 		# it worthwhile to do the split, otherwise, reduce the number of 
 		# splits to something more worthwhile
@@ -257,7 +251,7 @@ sub validate_or_request_dataset {
 	
 	# Process the reference dataset
 	$refDataSet = verify_or_request_feature_types(
-		'db'      => $ddb,
+		'db'      => $ddb || $Data->database,
 		'feature' => $refDataSet,
 		'prompt'  => "Enter the reference data set  ",
 		'single'  => 1,
@@ -269,7 +263,7 @@ sub validate_or_request_dataset {
 	
 	# Process the test dataset
 	$testDataSet = verify_or_request_feature_types(
-		'db'      => $ddb,
+		'db'      => $ddb || $Data->database,
 		'feature' => $testDataSet,
 		'prompt'  => "Enter the test data set  ",
 		'single'  => 1,
@@ -280,33 +274,12 @@ sub validate_or_request_dataset {
 }
 
 
-sub identify_indices {
-	# Identify columns in the input file
-	
-	my %index;
-	$index{name}   = find_column_index($mainData, "^name");
-	$index{type}   = find_column_index($mainData, "^type");
-	$index{id}     = find_column_index($mainData, "^primary_id");
-	$index{chrom}  = find_column_index($mainData, "^chr|seq");
-	$index{start}  = find_column_index($mainData, "^start");
-	$index{stop}   = find_column_index($mainData, "^stop|end");
-	$index{strand} = find_column_index($mainData, "strand");
-	unless (defined $index{id} or defined $index{name} or defined $index{chrom}) {
-		die " unable to identify at least name or chromosome column index!\n";
-	}
-	if ($set_strand and not defined $index{strand}) {
-		warn " unable to identify strand column to enforce strand!\n";
-		$set_strand = 0;
-	}
-	
-	return %index;
-}
-
-
-
 sub parallel_execution {
 	my $pm = Parallel::ForkManager->new($cpu);
 	
+	# Prepare new columns
+	my ($r_i, $p_i, $shift_i, $shiftr_i) = add_new_columns();
+
 	# generate base name for child processes
 	my $child_base_name = $outfile . ".$$"; 
 	
@@ -335,25 +308,18 @@ sub parallel_execution {
 		#### In child ####
 	
 		# splice the data structure
-		splice_data_structure($mainData, $i, $cpu);
+		$Data->splice_data($i, $cpu);
 		
 		# re-open database objects to make them clone safe
-		if ($database) {
-			$db = open_db_connection($database);
-		}
 		if ($data_database) {
-			$ddb = open_db_connection($data_database);
+			$ddb = open_db_connection($data_database, 1);
 		}
 		
-		# Prepare new columns
-		add_new_columns();
-	
 		# collect
-		my @results = collect_correlations();
+		my @results = collect_correlations($r_i, $p_i, $shift_i, $shiftr_i);
 
 		# write output file
-		my $success = write_tim_data_file(
-			'data'     => $mainData,
+		my $success = $Data->write_file(
 			'filename' => "$child_base_name.$i",
 			'gz'       => 0, # faster to write without compression
 		);
@@ -394,18 +360,17 @@ sub single_execution {
 	# run in a single thread
 	
 	# Prepare new columns
-	add_new_columns();
+	my ($r_i, $p_i, $shift_i, $shiftr_i) = add_new_columns();
 	
 	# collect
 	print " Collecting correlations....\n";
-	my @results = collect_correlations();
+	my @results = collect_correlations($r_i, $p_i, $shift_i, $shiftr_i);
 	
 	# summarize
 	summarize_results(@results);
 	
 	# write output file
-	my $success = write_tim_data_file(
-		'data'     => $mainData,
+	my $success = $Data->write_file(
 		'filename' => $outfile,
 		'gz'       => $gz,
 	);
@@ -420,6 +385,7 @@ sub single_execution {
 
 
 sub collect_correlations {
+	my ($r_i, $p_i, $shift_i, $shiftr_i) = @_; 
 	
 	# Set variables for summary analysis
 	my @correlations;
@@ -429,55 +395,28 @@ sub collect_correlations {
 	my $not_enough_data = 0;
 	my $no_variance = 0;
 	
-	
-	# set coordinate collection method
-	my $collect_coordinates;
-	if (defined $index{id} or (defined $index{name} and defined $index{type}) ) {
-		# using named features
-		# retrieve the coordinates from the database
-		$collect_coordinates = \&collect_coordinates_from_db;
+	# check that we can collect information
+	unless ($Data->feature_type eq 'named' or $Data->feature_type eq 'coordinate') {
+		die " Unable to identify the type of features in the input file." . 
+			" File must be a recognizable format and/or have column header names\n" . 
+			" for chromosome, start, stop; or named database features\n";
 	}
-	elsif (
-		defined $index{chrom} and 
-		defined $index{start} and 
-		defined $index{stop}
-	) {
-		# coordinates are defined in the input file
-		$collect_coordinates = \&collect_coordinates_from_file;
-	}
-	else {
-		# don't have enough info to continue
-		die " do not have enough information in the file to identify regions\n";
-	}
-	
 	
 	# Walk through features
-	for my $row (1 .. $mainData->{'last_row'}) {
-		
-		# Determine coordinates
-		my ($chromo, $start, $stop, $strand) = &{$collect_coordinates}($row);
-		unless ($chromo and $start and $stop) {
-			# verify coordinates
-			$mainData->{'data_table'}->[$row][$index{r}]       = '.';
-			if ($find_shift) {
-				$mainData->{'data_table'}->[$row][$index{shiftval}] = '.';
-				$mainData->{'data_table'}->[$row][$index{shiftr}]  = '.';
-			}
-			$not_enough_data++;
-			next;
-		}
+	my $stream = $Data->row_stream;
+	while (my $row = $stream->next_row) {
 		
 		
 		# determine reference point
 		my $ref_point;
 		if ($position == 5) {
-			$ref_point = $strand >= 0 ? $start : $stop;
+			$ref_point = $$row->strand >= 0 ? $row->start : $row->end;
 		}
 		elsif ($position == 4) {
-			$ref_point = int( ( ( $start + $stop ) / 2 ) + 0.5);
+			$ref_point = int( ( ( $row->start + $row->end ) / 2 ) + 0.5);
 		}
 		elsif ($position == 3) {
-			$ref_point = $strand >= 0 ? $stop : $start;
+			$ref_point = $row->strand >= 0 ? $row->end : $row->start;
 		}
 		
 		
@@ -488,25 +427,19 @@ sub collect_correlations {
 			# collecting data in a radius around a reference point
 			
 			# collect data
-			%ref_pos2data = get_region_dataset_hash(
-				'db'        => $db,
+			%ref_pos2data = $row->get_position_scores(
 				'ddb'       => $ddb,
 				'dataset'   => $refDataSet,
-				'chromo'    => $chromo,
 				'start'     => $ref_point - (2 * $radius),
 				'stop'      => $ref_point + (2 * $radius),
-				'strand'    => $strand,
 				'position'  => $position, 
 				'value'     => 'score',
 			);
-			%test_pos2data = get_region_dataset_hash(
-				'db'        => $db,
+			%test_pos2data = $row->get_position_scores(
 				'ddb'       => $ddb,
 				'dataset'   => $testDataSet,
-				'chromo'    => $chromo,
 				'start'     => $ref_point - (2 * $radius),
 				'stop'      => $ref_point + (2 * $radius),
-				'strand'    => $strand,
 				'position'  => $position, 
 				'value'     => 'score',
 			);
@@ -514,25 +447,15 @@ sub collect_correlations {
 		else {
 			# just collect data over the region
 			# collect data
-			%ref_pos2data = get_region_dataset_hash(
-				'db'        => $db,
+			%ref_pos2data = $row->get_position_scores(
 				'ddb'       => $ddb,
 				'dataset'   => $refDataSet,
-				'chromo'    => $chromo,
-				'start'     => $start,
-				'stop'      => $stop,
-				'strand'    => $strand,
 				'position'  => $position, 
 				'value'     => 'score',
 			);
-			%test_pos2data = get_region_dataset_hash(
-				'db'        => $db,
+			%test_pos2data = $row->get_position_scores(
 				'ddb'       => $ddb,
 				'dataset'   => $testDataSet,
-				'chromo'    => $chromo,
-				'start'     => $start,
-				'stop'      => $stop,
-				'strand'    => $strand,
 				'position'  => $position, 
 				'value'     => 'score',
 			);
@@ -553,11 +476,11 @@ sub collect_correlations {
 			)
 		) {
 			# not enough data points to work with
-			$mainData->{'data_table'}->[$row][$index{r}] = '.';
-			$mainData->{'data_table'}->[$row][$index{p}] = '.' if ($find_pvalue);
+			$row->value($r_i, '.');
+			$row->value($p_i, '.') if ($find_pvalue);
 			if ($find_shift) {
-				$mainData->{'data_table'}->[$row][$index{shiftval}] = '.';
-				$mainData->{'data_table'}->[$row][$index{shiftr}]  = '.';
+				$row->value($shift_i, '.');
+				$row->value($shiftr_i, '.');
 			}
 			$not_enough_data++;
 			next;
@@ -567,8 +490,7 @@ sub collect_correlations {
 		# Calculate Paired T-Test P value
 		# going to try this before normalizing
 		if ($find_pvalue) {
-			$mainData->{'data_table'}->[$row][$index{p}] = 
-				calculate_ttest(\%ref_pos2data, \%test_pos2data);
+			$row->value($p_i, calculate_ttest(\%ref_pos2data, \%test_pos2data) );
 		}
 		
 		
@@ -577,8 +499,8 @@ sub collect_correlations {
 		# especially when data is sparse
 		if ($interpolate) {
 			# pass the data reference and the region length
-			interpolate_values(\%ref_pos2data, ($stop - $start + 1));
-			interpolate_values(\%test_pos2data, ($stop - $start + 1));
+			interpolate_values(\%ref_pos2data, $row->length);
+			interpolate_values(\%test_pos2data, $row->length);
 		}
 		
 		
@@ -616,7 +538,7 @@ sub collect_correlations {
 			}
 		}
 		else {
-			for (my $i = 1; $i < ($stop - $start + 1); $i++) {
+			for (my $i = 1; $i < $row->length; $i++) {
 				# get values for each position, default 0
 				push @ref_data, $ref_pos2data{$i} || 0;
 				push @test_data, $test_pos2data{$i} || 0;
@@ -626,14 +548,12 @@ sub collect_correlations {
 		# verify the reference data
 		if ( min(@ref_data) == max(@ref_data) ) {
 			# no variance in the data! cannot calculate Pearson
-			$mainData->{'data_table'}->[$row][$index{r}] = '.';
-			$mainData->{'data_table'}->[$row][$index{p}] = '.' if ($find_pvalue);
+			$row->value($r_i, '.');
+			$row->value($p_i, '.') if ($find_pvalue);
 			if ($find_shift) {
-				$mainData->{'data_table'}->[$row][$index{shiftval}] = '.';
-				$mainData->{'data_table'}->[$row][$index{shiftr}]  = '.';
+				$row->value($shift_i, '.');
+				$row->value($shiftr_i, '.');
 			}
-			
-			# if the reference is no good, cannot continue, must bail
 			$no_variance++;
 			next;
 		}
@@ -654,9 +574,7 @@ sub collect_correlations {
 			# test has variance, calculate Pearson
 			$r = calculate_pearson(\@ref_data, \@test_data) || 0;
 		}
-		
-		# record
-		$mainData->{'data_table'}->[$row][$index{r}] = $r;
+		$row->value($r_i, $r);
 		push @correlations, $r if $r;
 		
 		
@@ -673,16 +591,15 @@ sub collect_correlations {
 			# change strand
 			if ($set_strand) {
 				# user is imposing a strand
-				if ($strand < 0) {
+				if ($$row->strand < 0) {
 					# only change shift direction if reverse strand
 					$best_shift *= -1;
 				}
 			}
 			
 			# record in data table
-			$mainData->{'data_table'}->[$row][$index{shiftval}] = $best_shift;
-			$mainData->{'data_table'}->[$row][$index{shiftr}] = 
-				$best_r > $r ? $best_r : '.';
+			$row->value($shift_i, $best_shift);
+			$row->value($shiftr_i, $best_r > $r ? $best_r : '.');
 			
 			# record for summary analyses
 			push @optimal_shifts, $best_shift if ($best_r > $r);
@@ -704,137 +621,54 @@ sub add_new_columns {
 	
 	# Required columns
 	# the Pearson column
-	my $r_i = $mainData->{'number_columns'};
-	$mainData->{$r_i} = {
-		'index'     => $r_i,
-		'name'      => 'Pearson_correlation',
-		'reference' => $refDataSet,
-		'test'      => $testDataSet,
-	};
-	$mainData->{'data_table'}->[0][$r_i] = 'Pearson_correlation';
-	$mainData->{'number_columns'} += 1;
+	my $r_i = $Data->add_column('Pearson_correlation');
+	$Data->metadata($r_i, 'reference', $refDataSet);
+	$Data->metadata($r_i, 'test', $testDataSet);
 	# extra information
-	$mainData->{$r_i}{'data_database'} = $data_database if ($data_database);
-	$mainData->{$r_i}{'normalization'} = $norm_method if ($norm_method);
-	$mainData->{$r_i}{'interpolate'}   = 1 if $interpolate;
+	$Data->metadata($r_i, 'data_database', $data_database) if $data_database;
+	$Data->metadata($r_i, 'normalization', $norm_method) if $norm_method;
+	$Data->metadata($r_i, 'interpolate', 1) if $interpolate;
 	if ($radius) {
-		$mainData->{$r_i}{'radius'} = $radius;
-		$mainData->{$r_i}{'reference_point'} = $position == 5 ? 
-			'5_prime' : $position == 4 ? 'midpoint' : '3_prime';
+		$Data->metadata($r_i, 'radius', $radius);
+		$Data->metadata($r_i, 'reference_point', $position == 5 ? 
+			'5_prime' : $position == 4 ? 'midpoint' : '3_prime');
 	}
 	
-	
 	# Optional columns if we're calculating an optimal shift
 	my ($shift_i, $shiftr_i);
 	if ($find_shift) {
 		
 		# optimal test-reference shift value
-		$shift_i = $mainData->{'number_columns'};
-		$mainData->{$shift_i} = {
-			'index'     => $shift_i,
-			'name'      => 'optimal_shift',
-			'reference' => $refDataSet,
-			'test'      => $testDataSet,
-			'radius'    => $radius,
-		};
-		$mainData->{'data_table'}->[0][$shift_i] = 'optimal_shift';
-		$mainData->{'number_columns'} += 1;
-		$mainData->{$shift_i}{'data_database'} = $data_database 
-			if ($data_database);
-		$mainData->{$shift_i}{'reference_point'} = $position == 5 ? 
-			'5_prime' : $position == 4 ? 'midpoint' : '3_prime';
+		$shift_i = $Data->add_column('optimal_shift');
+		$Data->metadata($shift_i, 'reference', $refDataSet);
+		$Data->metadata($shift_i, 'test', $testDataSet);
+		$Data->metadata($shift_i, 'radius', $radius);
+		$Data->metadata($shift_i, 'data_database', $data_database) if $data_database;
+		$Data->metadata($shift_i, 'reference_point', $position == 5 ? 
+			'5_prime' : $position == 4 ? 'midpoint' : '3_prime');
 		
 		# optimal test-reference shift value
-		$shiftr_i = $mainData->{'number_columns'};
-		$mainData->{$shiftr_i} = {
-			'index'     => $shiftr_i,
-			'name'      => 'shift_correlation',
-			'reference' => $refDataSet,
-			'test'      => $testDataSet,
-			'radius'    => $radius,
-		};
-		$mainData->{'data_table'}->[0][$shiftr_i] = 'shift_correlation';
-		$mainData->{'number_columns'} += 1;
-		$mainData->{$shiftr_i}{'data_database'} = $data_database 
-			if ($data_database);
-		$mainData->{$shiftr_i}{'reference_point'} = $position == 5 ? 
-			'5_prime' : $position == 4 ? 'midpoint' : '3_prime';
-		$mainData->{$shiftr_i}{'normalization'} = $norm_method if ($norm_method);
+		$shiftr_i = $Data->add_column('shift_correlation');
+		$Data->metadata($shiftr_i, 'reference', $refDataSet);
+		$Data->metadata($shiftr_i, 'test', $testDataSet);
+		$Data->metadata($shiftr_i, 'radius', $radius);
+		$Data->metadata($shiftr_i, 'data_database', $data_database) if $data_database;
+		$Data->metadata($shiftr_i, 'reference_point', $position == 5 ? 
+			'5_prime' : $position == 4 ? 'midpoint' : '3_prime');
+		$Data->metadata($shiftr_i, 'normalization', $norm_method) if $norm_method;
 	}
 	
 	# Paired Student T-Test P-value
 	my $p_i;
 	if ($find_pvalue) {
-		$p_i = $mainData->{'number_columns'};
-		$mainData->{$p_i} = {
-			'index'     => $p_i,
-			'name'      => 'ANOVA_Pval',
-			'reference' => $refDataSet,
-			'test'      => $testDataSet,
-			'transform' => '-Log10',
-		};
-		$mainData->{'data_table'}->[0][$p_i] = 'ANOVA_Pval';
-		$mainData->{'number_columns'} += 1;
+		$p_i = $Data->add_column('ANOVA_Pval');
+		$Data->metadata($p_i, 'reference', $refDataSet);
+		$Data->metadata($p_i, 'test', $testDataSet);
+		$Data->metadata($p_i, 'transform', '-Log10');
 	}
 	
 	# record the indices
-	$index{r} = $r_i;
-	$index{p} = $p_i;
-	$index{shiftval} = $shift_i;
-	$index{shiftr} = $shiftr_i;
-}
-
-
-sub collect_coordinates_from_db {
-	my $row = shift;
-	
-	# retrieve the coordinates from named features in the database
-	my $feature = get_feature(
-		'db'    => $db,
-		'id'    => defined $index{id} ? 
-			$mainData->{'data_table'}->[$row][$index{id}] : undef,
-		'name'  => defined $index{name} ? 
-			$mainData->{'data_table'}->[$row][$index{name}] : undef,
-		'type'  => defined $index{type} ? 
-			$mainData->{'data_table'}->[$row][$index{type}] : undef,
-	);
-	return unless ($feature);
-	
-	# calculate strand
-	my $strand;
-	if ($set_strand) {
-		$strand = $mainData->{'data_table'}->[$row][$index{strand}] =~ /-/ ?
-			-1 : 1;
-	}
-	else {
-		$strand = $feature->strand;
-	}
-	
-	# return coordinates
-	return ($feature->seq_id, $feature->start, $feature->end, $strand);
-}
-
-
-sub collect_coordinates_from_file {
-	my $row = shift;
-	
-	# calculate strand
-	my $strand;
-	if (defined $index{strand}) {
-		$strand = $mainData->{'data_table'}->[$row][$index{strand}] =~ /-/ ?
-			-1 : 1;
-	}
-	else {
-		$strand = 0;
-	}
-	
-	# return coordinates defined in the table
-	return (
-		$mainData->{'data_table'}->[$row][$index{chrom}], 
-		$mainData->{'data_table'}->[$row][$index{start}], 
-		$mainData->{'data_table'}->[$row][$index{stop}], 
-		$strand
-	);
+	return ($r_i, $p_i, $shift_i, $shiftr_i);
 }
 
 
